__title__ = 'mew'
__description__ = 'Python dataclass serializer/deserializer'
__url__ = 'http://github.com/cliffxuan/mew'
__version__ = '0.0.5'
__author__ = 'Cliff Xuan'
__author_email__ = 'cliff.xuan@gmail.com'
__license__ = 'MIT'
__copyright__ = 'Copyright 2019 Cliff Xuan'
